@@ -7,15 +7,15 @@ Net::RTSP::EventLoop - The Net::RTSP event loop
 =head1 SYNOPSIS
 
  use Net::RTSP::EventLoop;
- 
+
  my $event_loop = new Net::RTSP::EventLoop;
- 
+
  ...
- 
+
  $event_loop->add_socket($socket);
- 
+
  ...
- 
+
  $event_loop->run;
 
 =head1 DESCRIPTION
@@ -25,12 +25,19 @@ Net::RTSP::EventLoop - The Net::RTSP event loop
 use 5.005;
 use strict;
 use warnings;
-use Time::HiRes;
 use RTSP::Utility qw(get_named_params invoke_callback);
 use Net::RTSP::Socket;
 
 use constant DEFAULT_MAX_CONNECTIONS => 12;
 
+# we don't necessarily need Time::HiRes, but it would help:
+BEGIN {
+	eval {
+		require Time::HiRes;
+		Time::HiRes->import;
+	};
+}
+
 
 
 
@@ -156,7 +163,7 @@ sub execute_event_cycle
 		}
 
 		# notify the caller if their socket is readable:
-		if ($self->socket_is_readable($socket, $readable))
+		if ($self->is_socket_readable($socket, $readable))
 		{
 			$socket->readable_state;
 
@@ -164,7 +171,7 @@ sub execute_event_cycle
 		}
 
 		# notify the caller if their socket is writable:
-		if ($self->socket_is_writable($socket, $writable))
+		if ($self->is_socket_writable($socket, $writable))
 		{
 			$socket->writable_state;
 
@@ -175,7 +182,7 @@ sub execute_event_cycle
 	# the hooks to execute events from another event loop:
 	foreach my $hook (@{ $self->{loop_hooks} })
 	{
-		$self->{count}++ if (invoke_callback($hook));
+		$self->{count}++ if invoke_callback($hook);
 	}
 
 	return $self->{count} - $pre_event_count;
@@ -191,7 +198,7 @@ sub run
 
 	my $events_executed = 0;
 	
-	$events_executed++ while ($self->execute_event_cycle);
+	$events_executed++ while $self->execute_event_cycle;
 
 	return $events_executed;
 }
@@ -200,6 +207,67 @@ sub run
 
 
 
+sub after
+{
+	my ($self, $after_time, $after_event) = @_;
+
+	my $event = {
+		after_id    => $self->{next_after_id},
+		after_time  => $after_time,
+		after_event => $after_event,
+	};
+
+	push(@{ $self->{after_events} }, $event);
+
+	# order them so the next event to execute is at the beginning of the
+	# list:
+	@{ $self->{after_events} } = 
+		sort {$a->{after_time} <=> $b->{after_time}}
+			@{ $self->{after_events} };
+
+	$self->{next_after_id}++;
+
+	return $event->{after_id};
+}
+
+
+
+
+
+sub cancel_after_event
+{
+	my ($self, $after_id) = @_;
+
+	@{ $self->{after_events} } =
+		grep {$_->{after_id} != $after_id} @{ $self->{after_events} };
+}
+
+
+
+
+
+sub add_hook
+{
+	my ($self, $hook) = @_;
+
+	push(@{ $self->{hooks} }, $hook);
+}
+
+
+
+
+
+sub remove_hook
+{
+	my ($self, $hook) = @_;
+
+	@{ $self->{hooks} } = grep {$_ != $hook} @{ $self->{hooks} };
+}
+
+
+
+
+
 sub add_socket
 {
 	my ($self, $socket) = @_;
@@ -239,74 +307,26 @@ sub remove_socket
 
 
 
-sub add_after_event
-{
-	my ($self, $after_time, $after_event) = @_;
-
-	my $event = {
-		after_id    => $self->{next_after_id},
-		after_time  => $after_time,
-		after_event => $after_event,
-	};
-
-	push(@{ $self->{after_events} }, $event);
-
-	# order them so the next event to execute is at the beginning of the
-	# list:
-	@{ $self->{after_events} } = 
-		sort {$a->{after_time} <=> $b->{after_time}}
-			@{ $self->{after_events} };
-
-	$self->{next_after_id}++;
-
-	return $event->{after_id};
-}
-
-
-
-
-
-sub cancel_after_event
-{
-	my ($self, $after_id) = @_;
-
-}
-
-
-
-
-
-sub add_hook
-{
-	my ($self, $hook) = @_;
-
-	push(@{ $self->{hooks} }, $hook);
-}
-
-
-
-
-
-sub socket_is_readable
+sub is_socket_readable
 {
 	my ($self, $socket, $read_mask) = @_;
 
-	return unless ($socket->is_connected);
+	return unless $socket->is_connected;
 
-	return 1 if (vec($read_mask, fileno $socket->handle, 1));
+	return 1 if vec($read_mask, fileno $socket->handle, 1);
 }
 
 
 
 
 
-sub socket_is_writable
+sub is_socket_writable
 {
 	my ($self, $socket, $write_mask) = @_;
 
-	return unless ($socket->is_connected);
+	return unless $socket->is_connected;
 
-	return 1 if (vec($write_mask, fileno $socket->handle, 1));
+	return 1 if vec($write_mask, fileno $socket->handle, 1);
 }
 
 
