@@ -3,13 +3,15 @@ package Net::RTSP::Presentation;
 use 5.005;
 use strict;
 use warnings;
+use vars '@ISA';
 use URI;
-use Net::RTSP::Socket;
+use RTSP::Error;
 use RTSP::URI 'make_uri_absolute';
-use RTSP::Utility qw(get_named_params);
+use RTSP::Utility qw(get_named_params invoke_callback size_in_bytes);
+use Net::RTSP::Socket;
 
 # states for the presentation:
-use constant INACTIVE           => 0;
+use constant UNINITIALIZED      => 0;
 use constant INITIALIZING       => 1;
 use constant INITIALIZED        => 2;
 use constant SENDING_REQUEST    => 3; # client -> server
@@ -17,11 +19,16 @@ use constant RECEIVING_RESPONSE => 4; # client <- server
 use constant RECEIVING_REQUEST  => 5; # server -> client
 use constant SENDING_RESPONSE   => 6; # server <- client
 
+
+use constant DEFAULT_ACCEPT => 'application/sdp, application/rtsl, ' .
+                               'application/mheg';
+
 # we use smaller, more reasonable read sizes for the request/status line and
 # message header, since the default buffer size of 4096 is a little too large:
 use constant START_LINE_READ_SIZE => 128;
 use constant HEADER_READ_SIZE     => 1024;
 
+push(@ISA, qw(RTSP::Error));
 
 
 
@@ -39,15 +46,16 @@ sub new
 	# the hopefully full URI of the presentation:
 	my $presentation_uri = make_uri_absolute(shift);
 
-	my ($success_callback, $failure_callback,
+	my ($success_callback, $failure_callback, $server_request_callback,
 	    $timeout, $buffer_size, $pipelining);
 	get_named_params({
-		SuccessCallback => \$success_callback,
-		FailureCallback => \$failure_callback,
-		Timeout         => \$timeout,
-		BufferSize      => \$buffer_size,
-		Pipelining      => \$pipelining
-		}, @_
+		SuccessCallback       => \$success_callback,
+		FailureCallback       => \$failure_callback,
+		ServerRequestCallback => \$server_request_callback,
+		Timeout               => \$timeout,
+		BufferSize            => \$buffer_size,
+		Pipelining            => \$pipelining
+		}, \@_
 	);
 
 	# default to the Net::RTSP object's master settings:
@@ -63,12 +71,12 @@ sub new
 		($uri->scheme eq 'rtspu') ? 'UDP/IP' : 'TCP/IP';
 
 	# the socket to use for this presentation:
-	my $socket = new Net::RTSP::Socket;
+	my $socket = new Net::RTSP::Socket $transport_protocol;
 
 	# our Net::RTSP::Presentation object:
 	my $self = {
 		# the state of the presentation:
-		state                    => INACTIVE,
+		state                    => UNINITIALIZED,
 
 		# the Net::RTSP object:
 		rtsp                     => $rtsp,
@@ -77,7 +85,7 @@ sub new
 		'socket'                 => $socket,
 
 		# the absolute URI of the presentation:
-		presentation_uri         => $presentation_uri,
+		uri                      => $presentation_uri,
 
 		# the transport protocol to be used:
 		transport_protocol       => $transport_protocol,
@@ -100,7 +108,7 @@ sub new
 		failure_callback         => $failure_callback,
 
 		# the callback to invoke to process incoming requests:
-		server_request_callback  => $request_callback,
+		server_request_callback  => $server_request_callback,
 
 		# the number of seconds to timeout after:
 		timeout                  => $timeout,
@@ -111,11 +119,15 @@ sub new
 		# is RTSP request pipelining enabled?
 		pipelining               => $pipelining ? 1 : 0,
 
-		# the queue of pending request objects to dispatch for the
-		# event-driven interface:
+		# the queue of pending request objects to dispatch (used with
+		# the event-driven interface):
 		_pending_requests        => [],
 
-		# The list of active requests. requests get shifted out of the
+		# the list of pending responses to dispatch to the server (used
+		# with the event-driven interface):
+		_pending_responses       => [],
+
+		# The list of active requests. Requests get shifted out of the
 		# above list and pushed into this one:
 		_active_requests         => [],
 
@@ -146,7 +158,10 @@ sub new
 		# the number of bytes read so far for the request body/response
 		# content (checked against the value in the Content-Length
 		# header):
-		_message_body_bytes_read => 0
+		_message_body_bytes_read => 0,
+
+		# was the message body received yet?
+		_received_message_body   => 0,
 
 		# if the incoming message is a request message, this stores the
 		# RTSP::Request object for it:
@@ -159,7 +174,7 @@ sub new
 
 	bless($self, $class);
 
-	if ($self->is_event_driven)
+	if ($self->rtsp->is_event_driven)
 	{
 		# add the socket to the list of pending sockets and register
 		# callback to try to connect/initialize the socket as soon as
@@ -187,9 +202,15 @@ sub describe
 {
 	my $self = shift;
 
-	return $self->send_request(
-		new RTSP::Request (DESCRIBE => @_)
-	);
+	my $request = new RTSP::Request (DESCRIBE => @_);
+
+	$request->uri($self->uri) if ($request->uri eq '*');
+	$request->set_header(
+		Name   => 'Accept',
+		Accept => DEFAULT_ACCEPT
+	) unless $request->is_header_set('Accept');
+
+	return $self->send_request($request);
 }
 
 
@@ -226,11 +247,37 @@ sub setup_session
 {
 	my $self = shift;
 
-	my $session = new Net::RTSP::Session (@_);
+	my $request = new RTSP::Request (SETUP => @_);
+	my $uri     = $request->uri || $self->uri;
 
-	return $self->send_request(
-		
-	);
+	my $session = new Net::RTSP::Session ($self, $uri);
+	if ($self->presentation->is_event_driven)
+	{
+		$request->session_success_callback(sub {
+			my $response = shift;
+
+			my $session_id = $response->get_header('Session');
+
+			$session->id($session_id);
+			$session->ready;
+		});
+
+		$self->send_request($request);
+	}
+	else
+	{
+		my $response = $self->send_request($request);
+
+		if ($response)
+		{
+			my $session_id = $response->get_header('Session');
+
+			$session->id($session_id);
+			$session->ready;
+		}
+
+		return $response;
+	}
 }
 
 
@@ -267,7 +314,7 @@ sub send_request
 {
 	my ($self, $request) = @_;
 
-	if ($self->interface == EVENT_DRIVEN_INTERFACE)
+	if ($self->rtsp->is_event_driven)
 	{
 		push(@{ $self->{_pending_requests} }, $request);
 	}
@@ -279,8 +326,10 @@ sub send_request
 		# check if retransmission is needed:
 		if ($self->transport_protocol eq 'UDP/IP')
 		{
+			my $fd = fileno $self->socket->handle;
 			my $readable_mask = '';
-			vec($readable_mask, fileno $self->socket->handle, 1) =1;
+
+			vec($readable_mask, $fd, 1) = 1;
 			my $ready = select(
 				$readable_mask, undef, undef, $self->timeout
 			);
@@ -293,43 +342,46 @@ sub send_request
 
 
 
-		# get the status line:
+		# this stores the resulting response object:
 		my $response;
+
+		# get the status line:
 		while (1)
 		{
 			$self->_read_start_line or return;
 
-			return $self->raise_error(
-				'A message other than an RTSP ' .
-				'response was received from the server.'
-			) if ($self->_incoming_request
-				and not $self->_incoming_response);
+			if ($self->_received_start_line)
+			{
+				return $self->raise_error(
+					'A message other than an RTSP ' .
+					'response was received from the server.'
+				) if ($self->_incoming_request
+					or not $self->_incoming_response);
 
-			$response = $self->_incoming_response;
+				$response = $self->_incoming_response;
 
-			last;
+				last;
+			}
 		}
 
 		# get the response header:
 		while (1)
 		{
-			my $done;
-			my $bytes_read = $self->_read_message_header(\$done)
-				or return;
+			$self->_read_message_header or return;
 
-			last if $done;
+			last if $self->_received_message_header;
 		}
 
 		# now get the response body:
 		while (1)
 		{
-			my $done;
-			my $bytes_read = $self->_read_message_body(\$done)
-				or return;
+			$self->_read_message_body or return;
 
-			last if $done;
+			last if $self->_received_message_body;
 		}
 
+		$self->_clear_incoming_message;
+
 		return $response;
 	}
 }
@@ -342,24 +394,16 @@ sub send_response
 {
 	my ($self, $response) = @_;
 
-	if ($self->is_event_driven)
+	if ($self->rtsp->is_event_driven)
 	{
 		push(@{ $self->{_pending_responses} }, $response);
 	}
 	else
 	{
-		$self->state(SENDING_RESPONSE);
-		$self->_write_to_socket($response->as_string)
-			or return $self->raise_error(
-				sprintf("Couldn't send response to " .
-				        "server: %s.",
-					$self->socket->network_error
-				)
-			);
-		$self->state(INITIALIZED);
-
-		return 1;
+		$self->_dispatch_response($response) or return;
 	}
+
+	return 1;
 }
 
 
@@ -405,13 +449,13 @@ sub socket
 
 
 
-sub presentation_uri
+sub uri
 {
 	my $self = shift;
 
-	$self->{presentation_uri} = shift if @_;
+	$self->{uri} = shift if @_;
 
-	return $self->{presentation_uri};
+	return $self->{uri};
 }
 
 
@@ -535,24 +579,6 @@ sub pipelining
 
 
 
-sub is_event_driven
-{
-	return 1 if (shift->rtsp->interface == EVENT_DRIVEN_INTERFACE);
-}
-
-
-
-
-
-sub is_procedural
-{
-	return 1 if (shift->rtsp->interface == PROCEDURAL_INTERFACE);
-}
-
-
-
-
-
 sub succeed
 {
 	my $self = shift;
@@ -575,6 +601,21 @@ sub fail
 
 
 
+sub terminate
+{
+	my $self = shift;
+
+	return if ($self->state == UNINITIALIZED);
+
+	$self->socket->disconnect;
+	$self->rtsp->remove_socket($self->socket);
+	$self->state(UNINITIALIZED);
+}
+
+
+
+
+
 sub _non_blocking_initialization
 {
 	my $self = shift;
@@ -587,15 +628,15 @@ sub _non_blocking_initialization
 		sub {
 			$self->state(INITIALIZED);
 			$self->succeed;
-			$self->socket->register_writable(
+			$self->socket->register_writable_callback(
 				sub { $self->_dispatch_pending_messages }
 			);
-			$self->socket->register_readable(
+			$self->socket->register_readable_callback(
 				sub { $self->_process_incoming_message }
 			);
 		},
 		sub {
-			$self->state(INACTIVE);
+			$self->state(UNINITIALIZED);
 			$self->raise_error(
 				sprintf("Couldn't initialize socket: %s.",
 					$self->socket->network_error
@@ -630,7 +671,7 @@ sub _blocking_initialization
 	}
 	else
 	{
-		$self->state(INACTIVE);
+		$self->state(UNINITIALIZED);
 
 		return $self->raise_error(
 			sprintf("Couldn't initialize socket: %s.",
@@ -648,10 +689,17 @@ sub _dispatch_pending_messages
 {
 	my $self = shift;
 
-	return unless ($self->state == INITIALIZED)
+	return unless ($self->state == INITIALIZED);
 
-	# first, check for timeouts in any active
-	# request/response cycles:
+	# check for any outstanding responses to send to the server:
+	if ($self->_pending_responses)
+	{
+		my $response = shift @{ $self->{_pending_responses} };
+
+		$self->_dispatch_response($response);
+	}
+
+	# check for timeouts in any active request/response cycles:
 	foreach my $request (@{ $self->{_active_requests} })
 	{
 		# check for needed retransmission of a request if it
@@ -671,18 +719,27 @@ sub _dispatch_pending_messages
 		
 	}
 
-	if ($self->pipelining)
+	# send a pending request, or every pending request if pipelining is
+	# turned on:
+	if ($self->_pending_requests)
 	{
-		while (my $request = shift @{ $self->{_pending_requests} })
+		if ($self->pipelining)
 		{
-			$self->_dispatch_request($request) or last;
+			my @requests;
+			while (my $request =
+				shift @{ $self->{_pending_requests} })
+			{
+				push(@requests, $request);
+			}
+			
+			$self->_dispatch_requests(@requests);
 		}
-	}
-	else
-	{
-		my $request = shift @{ $self->{_pending_requests} };
+		else
+		{
+			my $request = shift @{ $self->{_pending_requests} };
 
-		$self->_dispatch_request($request);
+			$self->_dispatch_request($request);
+		}
 	}
 }
 
@@ -709,7 +766,7 @@ sub _dispatch_request
 			)
 		);
 
-		$failed_request->fail($self) if $self->is_event_driven;
+		$failed_request->fail($self) if $self->rtsp->is_event_driven;
 
 		return;
 	}
@@ -748,10 +805,11 @@ sub _dispatch_requests
 			)
 		);
 
+		# they all failed, all of them, not a single one can be used:
 		while (my $failed_request = shift @{$self->{_active_requests}})
 		{
 			$failed_request->fail($self)
-				if $self->is_event_driven;
+				if $self->rtsp->is_event_driven;
 		}
 
 		return;
@@ -764,6 +822,37 @@ sub _dispatch_requests
 
 
 
+sub _dispatch_response
+{
+	my ($self, $response) = shift;
+
+	$self->state(SENDING_RESPONSE);
+
+	$self->_prepare_response($response);
+
+	my $bytes_written = $self->_write_to_socket($response->as_string);
+	unless ($bytes_written)
+	{
+		$self->raise_error(
+			sprintf("Couldn't send response to server: %s.",
+				$self->socket->network_error
+			)
+		);
+
+		$response->fail($self) if $self->rtsp->is_event_driven;
+
+		return;
+	}
+
+	$response->succeed($self) if $self->rtsp->is_event_driven;
+
+	return $bytes_written;
+}
+
+
+
+
+
 sub _prepare_request
 {
 	my ($self, $request) = @_;
@@ -771,6 +860,9 @@ sub _prepare_request
 	# the sequence number to use for this request:
 	$self->_set_sequence_number($request);
 
+	# increment it for the next request:
+	$self->{cseq}++;
+
 	# add it to the list of active requests:
 	push(@{ $self->{_active_requests} }, $request);
 
@@ -780,6 +872,18 @@ sub _prepare_request
 
 
 
+sub _prepare_response
+{
+	my ($self, $response) = @_;
+
+	# the sequence number to use for this response:
+	$self->_set_sequence_number($response);
+}
+
+
+
+
+
 sub _process_incoming_message
 {
 	my $self = shift;
@@ -789,15 +893,19 @@ sub _process_incoming_message
 	{
 		if ($self->_received_message_header)
 		{
-			$self->_read_message_body(\$done) or return;
-			if ($done)
+			$self->_read_message_body or return;
+
+			if ($self->_received_message_body)
 			{
 				if ($self->state == RECEIVING_RESPONSE)
 				{
-					$self->_incoming_response->succeed(
+					my $request =
+					shift @{ $self->{_active_requests} };
+					
+					$request->succeed(
 						$self->_incoming_response,
 						$self
-					) if $self->is_event_driven;
+					) if $self->rtsp->is_event_driven;
 
 					$self->state(INITIALIZED)
 						unless $self->_active_requests;
@@ -811,19 +919,27 @@ sub _process_incoming_message
 				}
 				elsif ($self->state == RECEIVING_REQUEST)
 				{
+					# increment the sequence number for the
+					# next request we might send:
+					$self->{cseq}++;
+
+					invoke_callback(
+						$self->server_request_callback,
+						$self->_incoming_request
+					);
+
+					$self->_clear_incoming_message;
 				}
 			}
 		}
 		else
 		{
-			$self->_read_message_header(\$done) or return;
-			$slef->_received_message_header(1) if $done;
+			$self->_read_message_header or return;
 		}
 	}
 	else
 	{
-		$self->_read_start_line(\$done) or return;
-		$slef->_received_start_line(1) if $done;
+		$self->_read_start_line or return;
 	}
 }
 
@@ -836,7 +952,7 @@ sub _set_sequence_number
 	my ($self, $request) = @_;
 
 	my $cseq;
-	if ($request->header_is_set('CSeq'))
+	if ($request->is_header_set('CSeq'))
 	{
 		$cseq = $request->get_header('CSeq');
 	}
@@ -844,9 +960,6 @@ sub _set_sequence_number
 	{
 		$cseq = $self->cseq;
 
-		# increment it for the next request:
-		$self->{cseq}++;
-
 		# and set the CSeq header for it:
 		$request->set_header(
 			Name  => 'CSeq',
@@ -890,12 +1003,12 @@ sub _read_start_line
 			);
 		}
 
-		if ($self->state == RECEIVING_RESPONE)
+		if ($self->state == RECEIVING_RESPONSE)
 		{
 			my $failed_request = shift @{$self->{_active_requests}};
 
 			$failed_request->fail($self)
-				if $self->is_event_driven;
+				if $self->rtsp->is_event_driven;
 		}
 
 		return;
@@ -908,10 +1021,11 @@ sub _read_start_line
 		my ($line, $remainder) =
 			split(/\015\012/, $self->_incoming_start_line, 2);
 
-		$self->{_incoming_message_header} = $remainder;
+		# put back the rest:
+		$self->socket->unread($remainder);
 
-		my (@tokens) = split(/\s/, $line, 3);
-		my $version_pattern = qr/^RTSP/(\d+\.\d+)$/;
+		my (@tokens) = split(/ /, $line, 3);
+		my $version_pattern = qr/^RTSP\/(\d+\.\d+)$/;
 		if ($tokens[0] =~ /$version_pattern/)
 		{
 			my $response = new RTSP::Response (
@@ -922,18 +1036,33 @@ sub _read_start_line
 
 			$self->_incoming_response($response);
 		}
-		else
+		elsif ($tokens[2] =~ /$version_pattern/)
 		{
-			(my $version) = $tokens[2] =~ /$version_pattern/;
-
 			my $request = new RTSP::Request ($tokens[0],
 				URI     => $tokens[1],
-				Version => $version
+				Version => $1
 			);
 
 			$self->_incoming_request($request);
 			$self->state(RECEIVING_REQUEST);
 		}
+		else
+		{
+			$self->raise_error(
+				'A malformed message was sent; ignoring.'
+			);
+
+			if ($self->state == RECEIVING_RESPONSE)
+			{
+				my $failed_request =
+					shift @{$self->{_active_requests}};
+
+				$failed_request->fail($self)
+					if $self->rtsp->is_event_driven;
+			}
+		}
+
+		$self->_received_start_line(1);
 	}
 
 	return 1;
@@ -945,7 +1074,7 @@ sub _read_start_line
 
 sub _read_message_header
 {
-	my ($self, $is_done_ref) = @_;
+	my $self = shift;
 
 	my $bytes_read = $self->_read_from_socket(
 		\$self->{_incoming_message_header},
@@ -955,34 +1084,46 @@ sub _read_message_header
 
 	unless (defined $bytes_read)
 	{
-		$self->raise_error(
-			"Couldn't read the status/request line of " .
-			"the server's message: %s.",
-			$self->socket->network_error
-		);
-
-		if ($self->state == RECEIVING_RESPONE)
+		if ($self->state == RECEIVING_RESPONSE)
 		{
 			my $failed_request = shift @{$self->{_active_requests}};
 
+			$self->raise_error(
+				"The response header could not be read. %s.",
+				$self->socket->network_error
+			);
+
 			$failed_request->fail($self)
-				if $self->is_event_driven;
+				if $self->rtsp->is_event_driven;
+		}
+		elsif ($self->state == RECEIVING_REQUEST)
+		{
+			$self->raise_error(
+				"The request header could not be read. %s.",
+				$self->socket->network_error
+			);
 		}
 
 		return;
 	}
 
-	
-	if ($self->_incoming_message_header =~ /\015\012[^ \t]/)
+	$self->{_message_header_offset} += $bytes_read;
+
+	# check for completed header "Name: value" pairs:
+	while ($self->_incoming_message_header =~ /\015\012[^ \t]/)
 	{
-		# unfold the possibly folded header
+		# headers usually take up a single CRLF temrinated line:
 		my ($header, $remainder) =
 			split(/\015\012/, $self->{_incoming_message_header}, 2);
 
+		# Headers can be continued over multiple lines by starting
+		# subsiquint lines with a linear white space. So we see if the
+		# remainder after CRLF is a space, and continue the header if
+		# it is. (This is called "unfolding" a "folded" header.)
 		my $line;
-		while ($remainder =~ /^(\s)/)
+		while ($remainder =~ /^[ \t]/)
 		{
-			($line, $remainder) =~ split(/\015\012/, $remainder, 2);
+			($line, $remainder) = split(/\015\012/, $remainder, 2);
 
 			$header .= $line;
 		}
@@ -1005,15 +1146,26 @@ sub _read_message_header
 
 		if (substr($remainder, 0, 2) eq "\015\012")
 		{
-			# put the restr back, we're not ready for it yet:
+			# put the rest back onto the socket, we're not ready
+			# for it yet:
 			$self->socket->unread(substr($remainder, 2));
 
 			# but we are done:
-			$$is_done_ref = 1;
+			$self->_received_message_header(1);
+
+			last;
+		}
+		else
+		{
+			# put back the header, minus the one we just parsed:
+			$self->_incoming_message_header($remainder);
+			$self->_message_header_offset(
+				size_in_bytes($remainder)
+			);
 		}
 	}
 
-	return $bytes_read;
+	return 1;
 }
 
 
@@ -1022,7 +1174,7 @@ sub _read_message_header
 
 sub _read_message_body
 {
-	my ($self, $is_done_ref) = @_;
+	my $self = shift;
 
 	my $destination;
 	my $content_length;
@@ -1043,7 +1195,7 @@ sub _read_message_body
 
 	unless ($content_length)
 	{
-		$$is_done_ref = 1;
+		$self->_received_message_body(1);
 		return 1;
 	}
 
@@ -1065,19 +1217,19 @@ sub _read_message_body
 
 			$self->raise_error(
 				sprintf("Couldn't read response content from " .
-				        "server: %s.",
+				        "server. %s.",
 					$self->socket->network_error
 				)
 			);
 
 			$failed_request->fail($self)
-				if $self->is_event_driven;
+				if $self->rtsp->is_event_driven;
 		}
 		elsif ($self->state == RECEIVING_REQUEST)
 		{
 			$self->raise_error(
 				sprintf("Couldn't read request body from " .
-				        "server: %s.",
+				        "server. %s.",
 					$self->socket->network_error
 				)
 			);
@@ -1091,9 +1243,36 @@ sub _read_message_body
 	$self->{_message_body_bytes_read} += $bytes_read;
 	$self->{_message_body_offset}     += $bytes_read;
 
-	$$is_done = 1 if ($self->_message_body_bytes_read >= $content_length);
+	$self->_received_message_body(1)
+		if ($self->_message_body_bytes_read >= $content_length);
 
-	return $bytes_read;
+	return 1;
+}
+
+
+
+
+
+sub _pending_requests
+{
+	my $self = shift;
+
+	@{ $self->{_pending_requests} } = @_ if @_;
+
+	return @{ $self->{_pending_requests} };
+}
+
+
+
+
+
+sub _pending_responses
+{
+	my $self = shift;
+
+	@{ $self->{_pending_responses} } = @_ if @_;
+
+	return @{ $self->{_pending_responses} };
 }
 
 
@@ -1204,6 +1383,19 @@ sub _message_body_offset
 
 
 
+sub _received_message_body
+{
+	my $self = shift;
+
+	$self->{_received_message_body} = shift if @_;
+
+	return $self->{_received_message_body};
+}
+
+
+
+
+
 sub _incoming_response
 {
 	my $self = shift;
@@ -1244,6 +1436,7 @@ sub _clear_incoming_message
 
 	$self->_message_body_bytes_read(0);
 	$self->_message_body_offset(0);
+	$self->_received_message_body(0);
 
 	$self->_incoming_request(undef);
 	$self->_incoming_response(undef);
@@ -1253,12 +1446,12 @@ sub _clear_incoming_message
 
 
 
-
 sub _write_to_socket
 {
 	my ($self, $bytes_to_write) = @_;
 
-	if ($self->is_event_driven)
+	my $bytes_written;
+	if ($self->rtsp->is_event_driven)
 	{
 		$bytes_written = $self->socket->non_blocking_write(
 			$bytes_to_write,
@@ -1285,7 +1478,8 @@ sub _read_from_socket
 {
 	my ($self, $read_buffer_ref, $read_size, $offset) = @_;
 
-	if ($self->is_event_driven)
+	my $bytes_read;
+	if ($self->rtsp->is_event_driven)
 	{
 		$bytes_read = $self->socket->non_blocking_read(
 			$read_buffer_ref,
