@@ -17,6 +17,7 @@ use RTSP::Utility qw(
 	invoke_callback size_in_bytes get_os_name remove_package_prefix
 );
 
+# default values if no others are supplied:
 use constant DEFAULT_TIMEOUT   => 60;
 use constant DEFAULT_READ_SIZE => 4096;
 
@@ -38,7 +39,8 @@ use vars '@PROTOCOLS';
 $PROTOCOLS[SOCK_STREAM] = getprotobyname 'tcp',
 $PROTOCOLS[SOCK_DGRAM]  = getprotobyname 'udp';
 
-# we export them so the the socket state can be checked and set if needed:
+# we export the state constants so the the socket state can be checked and set
+# if needed:
 @EXPORT = qw(
 	DISCONNECTED
 	CONNECTABLE
@@ -57,13 +59,14 @@ push(@ISA, 'RTSP::Error');
 
 
 # thanks to Benjamin Goldberg for this hack to get the EINPROGRESS and
-# EWOULDBLOCK constants even on Windows (in addition to EINTR):
+# EWOULDBLOCK constants even on Windows (in addition to EINTR, which should be
+# available everywhere):
 BEGIN
 {
 	if (get_os_name() =~ /^MSWin/i)
 	{
 		require Errno;
-		import  Errno 'EINTR';
+		import  Errno qw(EINTR EAGAIN);
 
 		*EWOULDBLOCK = sub () { 10035 };
 		*EINPROGRESS = sub () { 10036 };
@@ -71,7 +74,7 @@ BEGIN
 	else
 	{
 		require Errno;
-		import  Errno qw(EINTR EWOULDBLOCK EINPROGRESS);
+		import  Errno qw(EINTR EAGAIN EWOULDBLOCK EINPROGRESS);
 	}
 }
 
@@ -120,6 +123,12 @@ sub new
 		# this stores a reference to the socket file handle:
 		handle                     => $socket,
 
+		# the hostname:
+		host                       => undef,
+
+		# the port number:
+		port                       => undef,
+
 		# this stores the type of socket this object encapsulates,
 		# either a stream (TCP/IP) or datagram (UDP/IP) socket, using
 		# the constants from Socket.pm:
@@ -130,7 +139,7 @@ sub new
 		last_active                => 0,
 
 		# the number of seconds to time a connect out afterwards:
-		connect_timeout            => DEFAULT_TIMEOUT,
+		connect_timeout            => 0,
 
 		# place store data the user "unreads" back into the socket if
 		# they aren't ready for it yet:
@@ -335,7 +344,7 @@ sub non_blocking_connect
 	my ($self, $host, $port, $timeout,
 	    $success_callback, $failure_callback) = @_;
 
-	$self->_make_socket_non_blocking
+	$self->_make_non_blocking
 		or return invoke_callback($failure_callback, $self);
 
 	$self->state(CONNECTING);
@@ -348,7 +357,9 @@ sub non_blocking_connect
 	unless ($rc or $! == EINPROGRESS or $! == EWOULDBLOCK)
 	{
 		$self->state(DISCONNECTED);
-		$self->network_error(remove_package_prefix($@));
+		$self->network_error(
+			"Couldn't connect to $host at port $port: $!"
+		);
 
 		return invoke_callback($failure_callback, $self);
 	}
@@ -379,28 +390,29 @@ sub blocking_connect
 {
 	my ($self, $host, $port, $timeout) = @_;
 
-	$self->_make_socket_non_blocking or return;
-
 	$self->state(CONNECTING);
 
 	my $rc = $self->_connect($host, $port);
 	unless ($rc or $! == EINPROGRESS)
 	{
 		$self->state(DISCONNECTED);
-		return $self->network_error(remove_package_prefix($@));
+		return $self->network_error(
+			"Couldn't connect to $host at $port: $!"
+		);
 	}
 
 	# now wait for it to become writable or timeout:
-	my $write_mask;
+	my $write_mask = '';
 	vec($write_mask, fileno $self->handle, 1) = 1;
 	if (select(undef, $write_mask, undef, $timeout))
 	{
 		$self->state(CONNECTED);
+		$self->_make_blocking;
 	}
 	else
 	{
 		$self->state(DISCONNECTED);
-		$self->network_error('Connect timed out.');
+		$self->network_error('Connect timed out');
 	}
 
 	return 1;
@@ -434,6 +446,7 @@ sub poll_non_blocking_connect
 	if ($self->is_connected)
 	{
 		$self->state(CONNECTED);
+		$self->_make_blocking;
 
 		invoke_callback($self->{connect_success_callback}, $self);
 	}
@@ -510,11 +523,7 @@ sub non_blocking_read
 		return $self->_read_unreaded_data(@_);
 	}
 
-	my $bytes_read = $self->_system_read(@_);
-
-	$self->last_active(time);
-
-	return $bytes_read;
+	return $self->_system_read(@_);
 }
 
 
@@ -564,7 +573,7 @@ sub blocking_read
 	my $timeout = pop || DEFAULT_TIMEOUT;
 
 	# block until the socket is readable:
-	my $read_mask;
+	my $read_mask = '';
 	vec($read_mask, fileno $self->handle, 1) = 1;
 	select($read_mask, undef, undef, $timeout);
 
@@ -658,11 +667,7 @@ sub non_blocking_write
 {
 	my $self = shift;
 
-	my $bytes_written = $self->_system_write(@_);
-
-	$self->last_active(time);
-
-	return $bytes_written;
+	return $self->_system_write(@_);
 }
 
 
@@ -699,7 +704,7 @@ sub blocking_write
 	my $timeout = $_[2];
 
 	# first, block until the socket is writable:
-	my $write_mask;
+	my $write_mask = '';
 	vec($write_mask, fileno $self->handle, 1) = 1;
 	select(undef, $write_mask, undef, $timeout);
 
@@ -786,7 +791,6 @@ sub is_connected
 # 		None.
 #
 
-
 sub state
 {
 	my $self = shift;
@@ -808,28 +812,49 @@ sub state
 ################################################################################
 #
 # 	Method
-# 		handle()
+# 		handle([$handle])
 #
 # 	Purpose
-# 		This method returns the IO::Socket::INET filehandle reference
-# 		for you to manipulate directly.
+# 		This method gets or sets the file handle of the socket used
+# 		internally.
 #
 # 	Parameters
-# 		None.
+# 		$handle - (Optional.) A socket file handle.
 #
 
 sub handle
 {
 	my $self = shift;
-	
-	if (@_)
-	{
-		$self->{handle} = shift;
-	}
-	else
-	{
-		return $self->{handle};
-	}
+
+	$self->{handle} = shift if @_;
+
+	return $self->{handle};
+}
+
+
+
+
+
+sub host
+{
+	my $self = shift;
+
+	$self->{host} = shift if @_;
+
+	return $self->{host};
+}
+
+
+
+
+
+sub port
+{
+	my $self = shift;
+
+	$self->{port} = shift if @_;
+
+	return $self->{port};
 }
 
 
@@ -846,9 +871,6 @@ sub handle
 # 		compared with the IO::Socket constants SOCK_STREAM and
 # 		SOCK_DGRAM.
 #
-# 	Parameters
-# 		None.
-#
 
 sub type { return shift->{type} }
 
@@ -859,33 +881,41 @@ sub type { return shift->{type} }
 ################################################################################
 #
 # 	Method
-# 		last_active()
+# 		last_active([$time])
 #
 # 	Purpose
-# 		This method returns the time() the socket was last active.
+# 		This method gets or sets the time() the socket was last active.
 #
 # 	Parameters
-# 		None.
+# 		$time - (Optional.) A new time() value of when the socket was
+# 		        last active.
 #
 
 sub last_active
 {
 	my $self = shift;
 
-	if (@_)
-	{
-		$self->{last_active} = shift;
-	}
-	else
-	{
-		return $self->{last_active};
-	}
+	$self->{last_active} = shift if @_;
+
+	return $self->{last_active};
 }
 
 
 
 
 
+################################################################################
+#
+# 	Method
+# 		network_error([$error])
+#
+# 	Purpose
+# 		This method gets or sets the last network error message.
+#
+# 	Parameters
+# 		$error - (Optional.) The last network error message.
+#
+
 sub network_error
 {
 	my $self = shift;
@@ -893,6 +923,11 @@ sub network_error
 	if (@_)
 	{
 		$self->{network_error} = shift;
+
+		# return nothing, so the caller can do
+		# "return $self->network_error($error_message);" and their sub
+		# will return correctly:
+		return;
 	}
 	else
 	{
@@ -904,15 +939,19 @@ sub network_error
 
 
 
+# this method performs a connect
 sub _connect
 {
 	my ($self, $host, $port) = @_;
 
+	$self->host($host);
+	$self->port($port);
+
 	# first get the packed four byte IP address:
 	my $address;
-	if ($host =~ /^\d+(\.\d+)+$/)
+	if ($host =~ /^\d+(\.\d+){3}$/)
 	{
-		$address = inet_aton($address);
+		$address = inet_aton($host);
 	}
 	else
 	{
@@ -927,7 +966,7 @@ sub _connect
 
 
 
-sub _make_socket_non_blocking
+sub _make_non_blocking
 {
 	my $self = shift;
 
@@ -961,11 +1000,45 @@ sub _make_socket_non_blocking
 
 
 
+sub _make_blocking
+{
+	my $self = shift;
+
+	unless (get_os_name() =~ /^MSWin/i)
+	{
+		my $flags = fcntl($self->handle, F_GETFL, 0)
+			or return $self->network_error(
+				"Couldn't get socket flags: $!"
+			);
+
+		until (fcntl($self->handle, F_SETFL, $flags & ~O_NONBLOCK))
+		{
+			return $self->network_error(
+				"Couldn't set socket to non-blocking mode: $!"
+			) unless ($! == EAGAIN or $! == EWOULDBLOCK);
+		}
+	}
+	else
+	{
+		my $mode = pack("L", 0);
+		ioctl($self->handle, FIONBIO, $mode)
+			or return $self->network_error(
+				"Couldn't set socket to non-blocking mode: $!"
+			);
+	}
+}
+
+
+
+
+
+# This reads data that was unread back onto the socket using unread():
 sub _read_unreaded_data
 {
 	my ($self, $read_buffer_ref, $read_size, $offset) = @_;
 
 	$read_size ||= DEFAULT_READ_SIZE;
+	$offset    ||= 0;
 
 	substr($$read_buffer_ref, $offset) =
 		substr($self->{unread_buffer}, 0, $read_size);
@@ -977,37 +1050,33 @@ sub _read_unreaded_data
 
 
 
+# This method performs a system-level read on the socket. It takes as its first
+# argument a reference to a scalar. The bytes read from the socket will be
+# stored there. As the second argument, it takes the number of bytes to read
+# from the socket into the buffer. As the third argument, it takes an offset of
+# where to start copying the bytes read into the buffer.
+#
+# It returns the number of bytes read or undef if some error occurs.
 sub _system_read
 {
 	my ($self, $read_buffer_ref, $read_size, $offset) = @_;
 
 	$read_size ||= DEFAULT_READ_SIZE;
+	$offset    ||= 0;
 
 	while (1)
 	{
 		$self->state(READING);
 
 		my $rv;
-		my $udp_recv_buffer;
 		if ($self->type == SOCK_STREAM)
 		{
-			if (defined $offset)
-			{
-				$rv = sysread(
-					$self->handle,
-					$$read_buffer_ref,
-					$read_size,
-					$offset
-				);
-			}
-			else
-			{
-				$rv = sysread(
-					$self->handle,
-					$$read_buffer_ref,
-					$read_size
-				);
-			}
+			$rv = sysread(
+				$self->handle,
+				$$read_buffer_ref,
+				$read_size,
+				$offset
+			);
 		}
 		else
 		{
@@ -1042,6 +1111,7 @@ sub _system_read
 		}
 
 		$self->state(CONNECTED);
+		$self->last_active(time);
 
 		unless (defined $rv)
 		{
@@ -1052,15 +1122,16 @@ sub _system_read
 			# a real network error occurred and there's nothing we
 			# can do about it:
 			return $self->network_error(
-				"Couldn't read from socket: $!."
+				"Couldn't read from socket: $!"
 			);
 		}
 
 		# sysread() returns the number of bytes actually read,
 		# recv()--which is actually recvfrom()--unfortunately, doesn't:
-		my $bytes_read = ($self->type eq SOCK_STREAM)
-					? $rv
-					: size_in_bytes($udp_recv_buffer);
+		my $bytes_read =
+			($self->type eq SOCK_STREAM)
+				? $rv
+				: size_in_bytes($$read_buffer_ref) - $offset;
 
 		return $bytes_read;
 	}
@@ -1070,6 +1141,13 @@ sub _system_read
 
 
 
+# This method performs a system-level write on the socket. It takes a scalar
+# containing bytes to write to the socket as its first argument, and as the
+# second optional argument it takes the number of bytes to write. If you don't
+# specify the number of bytes to write, then it will write all of the bytes in
+# the first argument to the socket.
+#
+# It returns the number of bytes written or undef if some error occurs.
 sub _system_write
 {
 	my $self         = shift;
@@ -1102,6 +1180,9 @@ sub _system_write
 			$bytes_written = $self->send($write_buffer, 0);
 		}
 
+		$self->state(CONNECTED);
+		$self->last_active(time);
+
 		unless (defined $bytes_written)
 		{
 			# try again if we were interrupted by SIGCHLD or
@@ -1110,13 +1191,11 @@ sub _system_write
 
 			# a real network error occurred and there's nothing we
 			# can do about it:
-			$self->network_error(
+			return $self->network_error(
 				"Couldn't write to socket: $!"
 			);
 		}
 
-		$self->state(CONNECTED);
-
 		# make sure the entire request was sent:
 		return $self->network_error(
 			sprintf('Data partially written (only %d %s of data ' .
@@ -1129,8 +1208,6 @@ sub _system_write
 			)
 		) unless ($bytes_written == $write_size);
 
-		$self->last_active(time);
-
 		return $bytes_written;
 	}
 }
