@@ -39,7 +39,7 @@ use strict;
 use warnings;
 use base 'RTSP::Error';
 use base 'RTSP::Headers';
-use RTSP::Utility 'get_named_params';
+use RTSP::Utility qw($CRLF get_named_params invoke_callback);
 use RTSP::URI 'make_uri_absolute';
 
 
@@ -52,8 +52,8 @@ use RTSP::URI 'make_uri_absolute';
 
 =head2 new(METHOD [, OPTIONS])
 
-The constructor method. This creates a new RTSP::Request object and returns a
-reference to it. The first and only mandatory argument is a string containing
+The constructor method. This creates a new B<RTSP::Request> object and returns
+a reference to it. The first and only mandatory argument is a string containing
 the RTSP request method name (for example, "SETUP", "OPTIONS", or "TEARDOWN").
 
 In addition, it takes the following optional named parameters:
@@ -62,13 +62,17 @@ In addition, it takes the following optional named parameters:
 
 =item URI/URL
 
-The URI (alternately spelled URL) of the RTSP request. The URI of an RTSP, if
-its present, must be absoluteDefaults to "*".
+The URI (alternately spelled URL) of the RTSP request. The request URI of an
+RTSP request, if its present, must be absolute. Defaults to "*".
 
 =item Version
 
 (Optional.) The RTSP protocol version number. Defaults to 1.0.
 
+=item Headers
+
+(Optional.) A hash reference containing header name/value pairs.
+
 =cut
 
 sub new
@@ -78,19 +82,27 @@ sub new
 
 	my $method = shift;
 
-	my ($uri, $url, $version, $headers, $body, $callback);
+	my ($uri, $url, $version, $headers, $body,
+	    $success_callback, $failure_callback,
+	    $session_success_callback, $session_failure_callback);
+
+	$uri = shift if (scalar @_ % 2);
+
 	get_named_params({
-		URI              => \$uri,
-		URL              => \$url,
-		Version          => \$version,
-		Headers          => \$headers,
-		Body             => \$body,
-		ResponseCallback => \$callback
+		URI                    => \$uri,
+		URL                    => \$url,
+		Version                => \$version,
+		Headers                => \$headers,
+		Body                   => \$body,
+		SuccessCallback        => \$success_callback,
+		FailureCallback        => \$failure_callback,
+		SessionSuccessCallback => \$session_success_callback,
+		SessionFailureCallback => \$session_failure_callback,
 		}, \@_
 	);
 
 	$uri     ||= $url;
-	$version ||= 1.0;
+	$version ||= '1.0';
 
 
 
@@ -99,33 +111,41 @@ sub new
 
 	# a string containing the request method ("DESCRIBE",
 	# "OPTIONS", "SETUP", or "PLAY", for example):
-	$self->{method}           = $method;
+	$self->{method}                   = $method;
 
 	# a string containing the absolute URI of the request or just a
 	# "*", indicating the request isn't specific to any one
 	# resource:
-	$self->{uri}              = make_uri_absolute($uri);
+	$self->{uri}                      = make_uri_absolute($uri);
 
 	# the protocol version number (probably 1.0):
-	$self->{version}          = $version;
+	$self->{version}                  = $version;
 
 	# the request body (probably nothing):
-	$self->{body}             = $body;
+	$self->{body}                     = $body;
 
 	# the callback to invoke if the request is sent successfully:
-	$self->{success_callback} = $success_callback;
+	$self->{success_callback}         = $success_callback;
 
-	# the callback to invoke if the request is unsuccessfull:
-	$self->{failure_callback} = $failure_callback;
+	# the callback to invoke if the request is unsuccessful:
+	$self->{failure_callback}         = $failure_callback;
+
+	# the callback to invoke for session requests if the request is
+	# successful:
+	$self->{session_success_callback} = $session_success_callback;
+
+	# the callback to invoke for session requests if the request is
+	# unsuccessful:
+	$self->{session_failure_callback} = $session_failure_callback;
 
 	# the time() the request was sent (used to ensure proper
 	# retransmission of requests over UDP if no response is
 	# received in the alloted time:
-	$self->{_time_sent}       = undef;
+	$self->{time_sent}                = undef;
 
 	# has the request been acknowledged by the server (meaning that
 	# the response is being received)?
-	$self->{_acknowledged}    = 0;
+	$self->{acknowledged}             = 0;
 
 
 
@@ -152,7 +172,9 @@ sub as_string
 	   $request_string .= $self->uri;
 	   $request_string .= ' RTSP/';
 	   $request_string .= $self->version;
+
 	   $request_string .= $CRLF;
+
 	   $request_string .= $self->headers_as_string;
 
 	   $request_string .= $CRLF;
@@ -236,6 +258,8 @@ sub body
 
 sub body_ref
 {
+	my $self = shift;
+
 	return \$self->{body}
 }
 
@@ -269,11 +293,38 @@ sub failure_callback
 
 
 
+sub session_success_callback
+{
+	my $self = shift;
+
+	$self->{session_success_callback} = shift if @_;
+
+	return $self->{session_success_callback};
+}
+
+
+
+
+
+sub session_failure_callback
+{
+	my $self = shift;
+
+	$self->{session_failure_callback} = shift if @_;
+
+	return $self->{session_failure_callback};
+}
+
+
+
+
+
 sub succeed
 {
 	my $self = shift;
 
 	invoke_callback($self->success_callback, @_);
+	invoke_callback($self->session_success_callback, @_);
 }
 
 
@@ -285,6 +336,7 @@ sub fail
 	my $self = shift;
 
 	invoke_callback($self->failure_callback, $self->error, @_);
+	invoke_callback($self->session_failure_callback, @_);
 }
 
 
